class YTStreamsClient:
    """
    YouTubeライブ配信関連の動画取得専用クラス
    """
    def __init__(self, youtube):
        """
        :param youtube: googleapiclient.discovery.buildで作成したYouTube APIインスタンス
        """
        self.youtube = youtube

    def get_upcoming_live_streams(self, channel_id: str, max_results: int = 10) -> list:
        """
        指定チャンネルの今後のライブ配信予定（未配信のライブ）一覧を取得する
        Returns: 動画情報リスト
        """
        from googleapiclient.errors import HttpError
        try:
            response = self.youtube.search().list(
                part='snippet',
                channelId=channel_id,
                type='video',
                eventType='upcoming',  # これで「これから始まるライブ」のみに絞れる
                maxResults=max_results,
                order='date'
            ).execute()
            return response.get('items', [])
        except HttpError as e:
            print(f"ライブ配信予定取得エラー: {e}")
            return []


import json
from datetime import datetime
from typing import Optional, Callable

class YTAnalysis:
    def __init__(self, channel_id: str, json_path: str):
        """
        :param channel_id: 監視したいYouTubeチャンネルID
        :param json_path: 増加率履歴を保存するJSONファイルのパス
        """
        self.channel_id = channel_id
        self.json_path = json_path
        self._fetched = False  # 初回fetch済みフラグ

    def fetch_current_subscribers(self, fetch_func: Callable[[str], int]) -> int:
        """
        fetch_funcを利用して現在の登録者数を取得し、jsonに保存する
        :param fetch_func: チャンネルIDを受け取って登録者数を返す関数
        :return: 取得した登録者数
        """
        count = fetch_func(self.channel_id)
        self._fetched = True
        self.record_subscriber_growth(count)
        return count

    def record_subscriber_growth(self, count: int, date: Optional[datetime] = None, subscribe_client=None):
        """
        現在の登録者数を記録し、前回との差分から増加率を計算して
        日付・登録者数・増加率をJSONファイルに追記保存する
        また subscribe_client (YTSubscribeClient) が渡された場合は履歴も追加する
        :param count: 現在の登録者数
        :param date: 日付（デフォルトは現在日時）
        :param subscribe_client: YTSubscribeClientのインスタンス（連携用）
        """
        if not self._fetched:
            raise RuntimeError("fetch_current_subscribersを先に実行してください")
        if date is None:
            date = datetime.now()
        date_str = date.strftime("%Y-%m-%d")

        # 既存データを読み込み
        try:
            with open(self.json_path, "r", encoding="utf-8") as f:
                data = json.load(f)
        except (FileNotFoundError, json.JSONDecodeError):
            data = []

        # 前回データがあれば増加率を算出
        if data:
            last = data[-1]
            last_count = last["subscribers"]
            last_date = datetime.strptime(last["date"], "%Y-%m-%d")
            days = (date - last_date).days or 1
            growth = (count - last_count) / days
        else:
            growth = 0  # 初回は増加率0

        # データを追記
        data.append({
            "channel_id": self.channel_id,
            "date": date_str,
            "subscribers": count,
            "growth_per_day": growth
        })

        # 保存
        with open(self.json_path, "w", encoding="utf-8") as f:
            json.dump(data, f, ensure_ascii=False, indent=2)

        # YTSubscribeClient との連携
        if subscribe_client is not None and hasattr(subscribe_client, "add_history"):
            subscribe_client.add_history(count, date)


import numpy as np
from datetime import datetime, timedelta
from typing import List, Tuple, Optional

class YTSubscribeClient:
    """
    YouTubeチャンネル登録者数の予測・解析用クライアント
    """
    def __init__(self):
        pass

    def predict_milestone_date(
        self, 
        history: List[Tuple[str, int]],  # 例: [("2025-06-01", 8000), ...]
        target: int
    ) -> Optional[dict]:
        """
        登録者履歴から、指定した登録者数（target）到達日を予測します。

        :param history: [("YYYY-MM-DD", 登録者数)] のリスト（古い順）
        :param target: 到達目標の登録者数
        :return: {"estimated_date": "YYYY-MM-DD", "days_left": 残り日数, "slope": 1日あたり増加数, "intercept": 切片} 予測不能時はNone
        """
        if len(history) < 2:
            return None

        dates = [datetime.strptime(d, "%Y-%m-%d") for d, _ in history]
        base_date = dates[0]
        days = np.array([(d - base_date).days for d in dates])
        subscribers = np.array([s for _, s in history])

        # 線形近似
        slope, intercept = np.polyfit(days, subscribers, 1)
        if slope <= 0:
            return None  # 伸びていない場合は予測不可

        days_to_target = (target - intercept) / slope
        estimated_date = base_date + timedelta(days=days_to_target)
        return {
            "estimated_date": estimated_date.strftime("%Y年%m月%d日"),
            "days_left": int(days_to_target),
            "slope": slope,
            "intercept": intercept
        }


from datetime import datetime
import pytz

class YTUtilsClient:

    # 日本時間（JST）に変換
    @staticmethod
    def to_jst(iso_timestamp, fmt="%Y-%m-%d %H:%M:%S"):
        """
        ISO形式のタイムスタンプをJSTに変換し、指定フォーマットで返す
        :param iso_timestamp: 例 '2025-06-12T10:00:00Z'
        :param fmt: 例 "%Y-%m-%d %H:%M:%S", "%Y/%m/%d %H:%M", "%m月%d日 %H:%M" など
        :return: フォーマット済みJST文字列
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        return dt.astimezone(jst).strftime(fmt)

    @staticmethod
    def to_jst_iso(iso_timestamp):
        """
        JSTのISO 8601形式（例: 2025-06-12T19:00:00+09:00）で返す
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        return dt.astimezone(jst).isoformat()

    @staticmethod
    def to_jst_short(iso_timestamp):
        """
        JSTの短い形式（例: 06/12 19:00）で返す
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        return dt.astimezone(jst).strftime("%m/%d %H:%M")

    @staticmethod
    def to_jst_ja(iso_timestamp):
        """
        日本語表記（例: 2025年06月12日 19時00分）で返す
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        return dt.astimezone(jst).strftime("%Y年%m月%d日 %H時%M分")

    @staticmethod
    def to_jst_weekday(iso_timestamp):
        """
        曜日付き（例: 2025-06-12(木) 19:00）で返す
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        w = ["月", "火", "水", "木", "金", "土", "日"]
        wd = w[dt.astimezone(jst).weekday()]
        return dt.astimezone(jst).strftime(f"%Y-%m-%d({wd}) %H:%M")

    @staticmethod
    def to_jst_timeonly(iso_timestamp):
        """
        時刻のみ（例: 19:00）で返す
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        return dt.astimezone(jst).strftime("%H:%M")

    @staticmethod
    def to_jst_custom(iso_timestamp, formatter):
        """
        JSTのdatetimeオブジェクトをユーザー定義の関数で任意の文字列に変換
        :param iso_timestamp: ISO形式のタイムスタンプ
        :param formatter: JSTのdatetimeを受け取り任意の文字列を返す関数
        :return: ユーザー定義のフォーマット文字列
        """
        dt = datetime.fromisoformat(iso_timestamp.replace('Z', '+00:00'))
        jst = pytz.timezone('Asia/Tokyo')
        return formatter(dt.astimezone(jst))


import os
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from typing import Optional, List, Dict, Any
from collections import Counter

class YTCommentClient:
    """
    YouTubeコメント取得・集計用クラス
    """
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv('YOUTUBE_API_KEY')
        if not self.api_key:
            raise ValueError("環境変数 YOUTUBE_API_KEY が設定されていません")
        self.youtube = build('youtube', 'v3', developerKey=self.api_key)

    def get_video_comments(self, video_id: str, max_results: int = 100) -> List[Dict[str, Any]]:
        comments = []
        next_page_token = None
        try:
            while True:
                response = self.youtube.commentThreads().list(
                    part='snippet,replies',
                    videoId=video_id,
                    maxResults=max_results,
                    pageToken=next_page_token
                ).execute()
                comments.extend(response['items'])
                next_page_token = response.get('nextPageToken')
                if not next_page_token:
                    break
            return comments
        except HttpError as e:
            print(f"コメント取得エラー: {e}")
            return []

    def count_comments_and_replies(self, video_id: str) -> Dict[str, Any]:
        comments = self.get_video_comments(video_id)
        total_comments = len(comments)
        total_replies = 0
        reply_authors = []
        for item in comments:
            reply_count = item['snippet']['totalReplyCount']
            if reply_count > 0 and 'replies' in item:
                for reply in item['replies']['comments']:
                    author = reply['snippet'].get('authorDisplayName', 'Unknown')
                    reply_authors.append(author)
                    total_replies += 1
        return {
            'total_comments': total_comments,
            'total_replies': total_replies,
            'unique_repliers': len(set(reply_authors)),
            'reply_authors': dict(Counter(reply_authors))
        }



import os
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from typing import Optional, Dict, Any, List

class YTVideoClient:
    """
    YouTube動画・チャンネル情報取得用クラス
    """
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv('YOUTUBE_API_KEY')
        if not self.api_key:
            raise ValueError("環境変数 YOUTUBE_API_KEY が設定されていません")
        self.youtube = build('youtube', 'v3', developerKey=self.api_key)

    def get_channel_info(self, channel_id: str) -> Optional[Dict[str, Any]]:
        try:
            response = self.youtube.channels().list(
                part='snippet,contentDetails',
                id=channel_id
            ).execute()
            if response['items']:
                return response['items'][0]
            return None
        except HttpError as e:
            print(f"チャンネル情報取得エラー: {e}")
            return None

    def get_latest_video(self, channel_id: str) -> Optional[Dict[str, Any]]:
        info = self.get_channel_info(channel_id)
        if not info:
            return None
        uploads_playlist_id = info['contentDetails']['relatedPlaylists']['uploads']
        try:
            playlist_response = self.youtube.playlistItems().list(
                part='snippet',
                playlistId=uploads_playlist_id,
                maxResults=1
            ).execute()
            if playlist_response['items']:
                return playlist_response['items'][0]['snippet']
            return None
        except HttpError as e:
            print(f"最新動画取得エラー: {e}")
            return None

    def get_video_statistics(self, video_id: str) -> Optional[Dict[str, Any]]:
        try:
            response = self.youtube.videos().list(
                part='statistics',
                id=video_id
            ).execute()
            if response['items']:
                return response['items'][0]['statistics']
            return None
        except HttpError as e:
            print(f"動画統計取得エラー: {e}")
            return None

    def get_videos_by_type(
        self, channel_id: str, max_results: int = 50
    ) -> Dict[str, List[Dict[str, Any]]]:
        """
        指定チャンネルの動画を「ライブ配信アーカイブ」「通常動画」「Shorts」に分類して取得する

        Returns:
            {
                "live_archives": [...],
                "normal_videos": [...],
                "shorts": [...]
            }
        """
        import isodate

        def duration_to_seconds(duration: str) -> int:
            try:
                return int(isodate.parse_duration(duration).total_seconds())
            except Exception:
                return 0

        # まずアップロード動画のplaylistIdを取得
        channel_info = self.get_channel_info(channel_id)
        if not channel_info:
            return {"live_archives": [], "normal_videos": [], "shorts": []}
        uploads_playlist_id = channel_info['contentDetails']['relatedPlaylists']['uploads']

        # プレイリストから動画ID一覧を取得
        video_ids = []
        next_page_token = None
        while True:
            try:
                playlist_response = self.youtube.playlistItems().list(
                    part='snippet',
                    playlistId=uploads_playlist_id,
                    maxResults=min(max_results, 50),
                    pageToken=next_page_token
                ).execute()
            except HttpError as e:
                print(f"プレイリスト取得エラー: {e}")
                break
            for item in playlist_response.get('items', []):
                video_ids.append(item['snippet']['resourceId']['videoId'])
                if len(video_ids) >= max_results:
                    break
            next_page_token = playlist_response.get('nextPageToken')
            if not next_page_token or len(video_ids) >= max_results:
                break

        # 動画情報を一括取得
        live_archives = []
        normal_videos = []
        shorts = []
        for i in range(0, len(video_ids), 50):
            batch_ids = video_ids[i:i+50]
            try:
                videos_response = self.youtube.videos().list(
                    part='snippet,contentDetails,liveStreamingDetails',
                    id=','.join(batch_ids)
                ).execute()
            except HttpError as e:
                print(f"動画情報取得エラー: {e}")
                continue
            for video in videos_response.get('items', []):
                snippet = video.get('snippet', {})
                content_details = video.get('contentDetails', {})
                live_details = video.get('liveStreamingDetails', {})
                duration = content_details.get('duration', '')
                title = snippet.get('title', '').lower()
                width = snippet.get('thumbnails', {}).get('maxres', {}).get('width') or \
                        snippet.get('thumbnails', {}).get('standard', {}).get('width') or \
                        snippet.get('thumbnails', {}).get('high', {}).get('width')
                height = snippet.get('thumbnails', {}).get('maxres', {}).get('height') or \
                         snippet.get('thumbnails', {}).get('standard', {}).get('height') or \
                         snippet.get('thumbnails', {}).get('high', {}).get('height')

                # ライブ配信（アーカイブ含む）判定
                if live_details:
                    live_archives.append(video)
                # Shorts判定（60秒以下＆縦長 or タイトルに#shorts）
                elif (
                    duration and duration_to_seconds(duration) <= 60 and
                    ((height and width and height > width) or '#shorts' in title)
                ):
                    shorts.append(video)
                else:
                    normal_videos.append(video)

        return {
            "live_archives": live_archives,
            "normal_videos": normal_videos,
            "shorts": shorts
        }


import ytwrap

CHANNEL_ID = "UCYrHEsf7bhiFJssY5btS7Zg"

def main():
    video_client = ytwrap.YTVideoClient()
    comment_client = ytwrap.YTCommentClient()

    # 最新動画取得
    latest_video = video_client.get_latest_video(CHANNEL_ID)
    if not latest_video:
        print("動画が見つかりませんでした")
        return
    video_id = latest_video['resourceId']['videoId']
    video_title = latest_video['title']
    published_at = latest_video['publishedAt']

    # 動画統計
    video_stats = video_client.get_video_statistics(video_id)
    api_comment_count = int(video_stats.get('commentCount', 0)) if video_stats else 0

    # コメント集計
    comment_stats = comment_client.count_comments_and_replies(video_id)

    print(f"動画タイトル: {video_title}")
    print(f"公開日: {published_at}")
    print(f"API取得コメント総数: {api_comment_count}")
    print(f"実際に分析したコメント数: {comment_stats['total_comments']}")
    print(f"総返信数: {comment_stats['total_replies']}")
    print(f"返信者数: {comment_stats['unique_repliers']}")
    print(f"返信者ランキングTOP3: {sorted(comment_stats['reply_authors'].items(), key=lambda x: x[1], reverse=True)[:3]}")

if __name__ == "__main__":
    main()



from .video import YTVideoClient
from .comment import YTCommentClient
from .utils import YTUtilsClient
from .channel import *
from .streams import YTStreamsClient
from .subscribe import YTSubscribeClient
from .analysi import YTAnalysis

__all__ = ["YTVideoClient", "YTCommentClient", "YTUtilsClient", "YTStreamsClient", "YTSubscribeClient", "YTAnalysis"]



from setuptools import setup, find_packages

with open("pypi.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="ytwrap",
    version="0.2.1",
    description="Python wrapper for the YouTube Data API v3.",
    author="Himarry",
    url="https://github.com/Himarry/ytwrap",
    project_urls={
        "Source": "https://github.com/Himarry/ytwrap",
        "Tracker": "https://github.com/Himarry/ytwrap/issues",
    },
    packages=find_packages(),
    install_requires=[
        "google-api-python-client",
        "isodate",
        "pytz"
    ],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)



# 指定ディレクトリ配下の全pyファイルの全関数を自動テストし、結果を明示的に表示するスクリプト
import os
import sys
import importlib
import inspect
import traceback

TARGET_DIR = "ytwrap"


def test_all_functions_in_module(module, modname):
    results = []
    for name, obj in inspect.getmembers(module):
        if inspect.isfunction(obj) and obj.__module__ == module.__name__:
            try:
                sig = inspect.signature(obj)
                # 引数なしで呼び出せる場合のみ実行
                if all(
                    p.default != inspect.Parameter.empty
                    or p.kind
                    in (inspect.Parameter.VAR_POSITIONAL, inspect.Parameter.VAR_KEYWORD)
                    for p in sig.parameters.values()
                ):
                    result = obj()
                    results.append(
                        f"{modname}の{name}関数: 正常に動作しました。結果: {result}"
                    )
                elif name == "duration_to_seconds":
                    result = obj("PT59S")
                    results.append(
                        f"{modname}の{name}関数: 正常に動作しました。結果: {result}"
                    )
                elif name == "classify_youtube_videos":
                    test_data = [
                        {
                            "snippet": {"title": "Test #shorts"},
                            "contentDetails": {"duration": "PT59S"},
                            "player": {"width": 720, "height": 1280},
                            "liveStreamingDetails": {},
                        }
                    ]
                    result = obj(test_data)
                    results.append(
                        f"{modname}の{name}関数: 正常に動作しました。結果: {result}"
                    )
                else:
                    results.append(f"{modname}の{name}関数: スキップ（引数が必要です）")
            except Exception as e:
                results.append(
                    f"{modname}の{name}関数が正常に動作しませんでした。エラー: {e}\n{traceback.format_exc()}"
                )
    return results


def main():
    sys.path.insert(0, os.getcwd())
    file_func_map = {}
    for fname in os.listdir(TARGET_DIR):
        if fname.endswith(".py") and not fname.startswith("__"):
            modname = f"{TARGET_DIR}.{fname[:-3]}"
            try:
                module = importlib.import_module(modname)
                func_names = []
                # モジュール直下の関数
                for name, obj in inspect.getmembers(module):
                    if inspect.isfunction(obj) and obj.__module__ == module.__name__:
                        func_names.append(name)
                # クラス内のメソッド
                for cname, cls in inspect.getmembers(module, inspect.isclass):
                    if cls.__module__ == module.__name__:
                        for mname, mobj in inspect.getmembers(cls):
                            if inspect.isfunction(mobj) or inspect.ismethod(mobj):
                                if not mname.startswith("__"):
                                    func_names.append(f"{cname}.{mname}")
                if func_names:
                    file_func_map[fname] = func_names
            except Exception:
                pass
    for fname, funcs in file_func_map.items():
        print(f"{fname}：")
        for func in funcs:
            # クラス名.メソッド名ならメソッド名だけ抽出
            if "." in func:
                short_name = func.split(".", 1)[1]
            else:
                short_name = func
            # テスト実行して色分け
            try:
                modname = f"{TARGET_DIR}.{fname[:-3]}"
                module = importlib.import_module(modname)
                # クラスメソッドかグローバル関数かで取得
                if "." in func:
                    cname, mname = func.split(".", 1)
                    cls = getattr(module, cname)
                    method = getattr(cls, mname)
                    # __init__以外はダミー引数で呼び出しテスト（引数必須はスキップ）
                    if mname == "__init__":
                        continue
                    sig = inspect.signature(method)
                    if all(
                        p.default != inspect.Parameter.empty
                        or p.kind
                        in (
                            inspect.Parameter.VAR_POSITIONAL,
                            inspect.Parameter.VAR_KEYWORD,
                        )
                        or p.name == "self"
                        for p in sig.parameters.values()
                    ):
                        # インスタンス化してself付きで呼ぶ
                        inst = (
                            cls(api_key="dummy")
                            if "api_key" in sig.parameters
                            else cls()
                        )
                        method(inst)
                        print(f"　・\033[32m{short_name}\033[0m")
                    else:
                        print(f"　・\033[32m{short_name}\033[0m (引数省略)")
                else:
                    fn = getattr(module, func)
                    sig = inspect.signature(fn)
                    if all(
                        p.default != inspect.Parameter.empty
                        or p.kind
                        in (
                            inspect.Parameter.VAR_POSITIONAL,
                            inspect.Parameter.VAR_KEYWORD,
                        )
                        for p in sig.parameters.values()
                    ):
                        fn()
                        print(f"　・\033[32m{short_name}\033[0m")
                    elif func == "duration_to_seconds":
                        fn("PT59S")
                        print(f"　・\033[32m{short_name}\033[0m")
                    elif func == "classify_youtube_videos":
                        test_data = [
                            {
                                "snippet": {"title": "Test #shorts"},
                                "contentDetails": {"duration": "PT59S"},
                                "player": {"width": 720, "height": 1280},
                                "liveStreamingDetails": {},
                            }
                        ]
                        fn(test_data)
                        print(f"　・\033[32m{short_name}\033[0m")
                    else:
                        print(f"　・\033[32m{short_name}\033[0m (引数省略)")
            except Exception:
                print(f"　・\033[31m{short_name}\033[0m (エラー)")
        print("")


if __name__ == "__main__":
    main()



from datetime import datetime
import os
import json

import ytwrap  # ← ここだけでOK

TEST_JSON = "test_growth_history.json"
if os.path.exists(TEST_JSON):
    os.remove(TEST_JSON)

CHANNEL_ID = "UC_TESTCHANNELID"
test_data = [
    (datetime(2025, 6, 10), 8000),
    (datetime(2025, 6, 11), 8500),
    (datetime(2025, 6, 12), 9000),
]

ana = ytwrap.YTAnalysis(CHANNEL_ID, TEST_JSON)
sub_client = ytwrap.YTSubscribeClient(CHANNEL_ID)

for dt, cnt in test_data:
    ana.record_subscriber_growth(cnt, date=dt, subscribe_client=sub_client)

print("---- YTSubscribeClient履歴 ----")
for h in sub_client.history:
    print(h)

result = sub_client.predict_milestone_date(10000)
print("\n---- 1万人到達予測 ----")
if result:
    print(f"予想到達日: {result['estimated_date']}（あと{result['days_left']}日）")
    print(f"増加率（1日あたり）: {result['slope']:.2f}")
else:
    print("予測できませんでした")

print("\n---- 保存JSON内容 ----")
with open(TEST_JSON, "r", encoding="utf-8") as f:
    print(json.dumps(json.load(f), ensure_ascii=False, indent=2))


import re
import sys
import subprocess
import shutil
import os

SETUP_PATH = "setup.py"

# バージョン番号を種別ごとに上げる
# kind: 'patch', 'minor', 'major'
def bump_version(version: str, kind: str) -> str:
    parts = [int(p) for p in version.strip().split(".")]
    if kind == "patch":
        parts[2] += 1
    elif kind == "minor":
        parts[1] += 1
        parts[2] = 0
    elif kind == "major":
        parts[0] += 1
        parts[1] = 0
        parts[2] = 0
    else:
        raise ValueError("kindはpatch, minor, majorのいずれか")
    return f"{parts[0]}.{parts[1]}.{parts[2]}"

def main():
    print("バージョンアップの種類を選択してください:")
    print("1: patch (例: 0.1.0 → 0.1.1)")
    print("2: minor (例: 0.1.0 → 0.2.0)")
    print("3: major (例: 0.1.0 → 1.0.0)")
    kind_map = {"1": "patch", "2": "minor", "3": "major"}
    kind = input("番号を入力: ").strip()
    if kind not in kind_map:
        print("無効な選択です")
        sys.exit(1)
    kind = kind_map[kind]

    with open(SETUP_PATH, encoding="utf-8") as f:
        setup_code = f.read()

    m = re.search(r'version\s*=\s*["\']([0-9]+\.[0-9]+\.[0-9]+)["\']', setup_code)
    if not m:
        print("setup.py からバージョン番号を検出できませんでした")
        sys.exit(1)
    old_version = m.group(1)
    new_version = bump_version(old_version, kind)

    new_code = re.sub(
        r'(version\s*=\s*["\'])[0-9]+\.[0-9]+\.[0-9]+(["\'])',
        r'\g<1>' + new_version + r'\2',
        setup_code
    )

    with open(SETUP_PATH, "w", encoding="utf-8") as f:
        f.write(new_code)

    print(f"バージョンを {old_version} → {new_version} に更新しました")

    # dist, build, egg-infoを削除
    for folder in ["dist", "build", "ytwrap.egg-info"]:
        if os.path.exists(folder):
            print(f"{folder} を削除します...")
            shutil.rmtree(folder)

    # パッケージビルド
    print("パッケージをビルド中...")
    subprocess.run([sys.executable, "setup.py", "sdist", "bdist_wheel"], check=True)

    # PyPIにアップロード
    print("PyPIにアップロードします...")
    subprocess.run(["twine", "upload", "dist/*"])

if __name__ == "__main__":
    main()



from ytwrap.subscribe import YTSubscribeClient

# 対象チャンネル: UCYrHEsf7bhiFJssY5btS7Zg
history = [
    ("2025-05-01", 3200),
    ("2025-05-10", 3400),
    ("2025-05-20", 3700),
    ("2025-06-01", 4100),
    ("2025-06-10", 4500),
]

# 目標登録者数
TARGET = 5000

client = YTSubscribeClient()
result = client.predict_milestone_date(history, TARGET)

if result:
    print(f"登録者{TARGET}人突破予測日: {result['estimated_date']}")
    print(f"あと{result['days_left']}日 (1日あたり増加数: {result['slope']:.2f})")
else:
    print("予測できませんでした（登録者数が増加していない、または履歴が不足）")
